@@ -1,34 +1,42 @@
-from collections import defaultdict
-from warnings import warn, catch_warnings, simplefilter
-from decimal import Decimal
-from ast import Name, Or, And, BoolOp
-from gzip import GzipFile
-from bz2 import BZ2File
-from tempfile import NamedTemporaryFile
-from sys import exc_info
+# -*- coding: utf-8 -*-
+
+from __future__ import absolute_import
+
 import re
+from ast import And, BoolOp, Name, Or
+from bz2 import BZ2File
+from collections import defaultdict
+from decimal import Decimal
+from gzip import GzipFile
+from tempfile import NamedTemporaryFile
+from warnings import catch_warnings, simplefilter, warn
 
 from six import iteritems, string_types
 
-from .. import Metabolite, Reaction, Gene, Model
-from ..core.Gene import parse_gpr
-from ..manipulation.modify import _renames
-from ..manipulation.validate import check_reaction_bounds, \
-    check_metabolite_compartment_formula
+from cobra.core import Configuration, Gene, Metabolite, Model, Reaction
+from cobra.core.gene import parse_gpr
+from cobra.manipulation.modify import _renames
+from cobra.manipulation.validate import check_metabolite_compartment_formula
+from cobra.util.solver import set_objective
+
 
 try:
-    from lxml.etree import parse, Element, SubElement, \
-        ElementTree, register_namespace, ParseError, XPath
+    from lxml.etree import (
+        parse, Element, SubElement, ElementTree, register_namespace,
+        ParseError, XPath)
+
     _with_lxml = True
 except ImportError:
-    warn("Install lxml for faster SBML I/O")
     _with_lxml = False
     try:
-        from xml.etree.cElementTree import parse, Element, SubElement, \
-            ElementTree, register_namespace, ParseError
+        from xml.etree.cElementTree import (
+            parse, Element, SubElement, ElementTree, register_namespace,
+            ParseError)
     except ImportError:
-        from xml.etree.ElementTree import parse, Element, SubElement, \
-            ElementTree, register_namespace, ParseError
+        XPath = None
+        from xml.etree.ElementTree import (
+            parse, Element, SubElement, ElementTree, register_namespace,
+            ParseError)
 
 # use sbml level 2 from sbml.py (which uses libsbml). Eventually, it would
 # be nice to use the libSBML converters directly instead.
@@ -37,16 +45,18 @@ try:
 except ImportError:
     libsbml = None
 else:
-    from .sbml import create_cobra_model_from_sbml_file as read_sbml2
-    from .sbml import write_cobra_model_to_sbml_file as write_sbml2
+    from cobra.io.sbml import create_cobra_model_from_sbml_file as read_sbml2
+    from cobra.io.sbml import write_cobra_model_to_sbml_file as write_sbml2
 
 try:
-    from sympy import Basic
-except:
+    from optlang.symbolics import Basic
+except ImportError:
     class Basic:
         pass
 
 
+CONFIGURATION = Configuration()
+
 # deal with namespaces
 namespaces = {"fbc": "http://www.sbml.org/sbml/level3/version1/fbc/version2",
               "sbml": "http://www.sbml.org/sbml/level3/version1/core",
@@ -62,6 +72,7 @@ def ns(query):
         query = query.replace(prefix + ":", "{" + uri + "}")
     return query
 
+
 # XPATH query wrappers
 fbc_prefix = "{" + namespaces["fbc"] + "}"
 sbml_prefix = "{" + namespaces["sbml"] + "}"
@@ -82,6 +93,9 @@ SPECIES_XPATH = ns("sbml:listOfSpecies/sbml:species[@boundaryCondition='%s']")
 OBJECTIVES_XPATH = ns("fbc:objective[@fbc:id='%s']/"
                       "fbc:listOfFluxObjectives/"
                       "fbc:fluxObjective")
+LONG_SHORT_DIRECTION = {'maximize': 'max', 'minimize': 'min'}
+SHORT_LONG_DIRECTION = {'min': 'minimize', 'max': 'maximize'}
+
 if _with_lxml:
     RDF_ANNOTATION_XPATH = ("sbml:annotation/rdf:RDF/"
                             "rdf:Description[@rdf:about=$metaid]/"
@@ -99,8 +113,8 @@ else:
         search_xpath = RDF_ANNOTATION_XPATH % metaid
         for i in sbml_element.iterfind(search_xpath):
             yield get_attrib(i, "rdf:resource")
-        for i in sbml_element.iterfind(search_xpath
-                                       .replace("isEncodedBy", "is")):
+        for i in sbml_element.iterfind(search_xpath.replace(
+                "isEncodedBy", "is")):
             yield get_attrib(i, "rdf:resource")
 
 
@@ -112,7 +126,7 @@ def get_attrib(tag, attribute, type=lambda x: x, require=False):
     value = tag.get(ns(attribute))
     if require and value is None:
         msg = "required attribute '%s' not found in tag '%s'" % \
-                             (attribute, tag.tag)
+              (attribute, tag.tag)
         if tag.get("id") is not None:
             msg += " with id '%s'" % tag.get("id")
         elif tag.get("name") is not None:
@@ -185,7 +199,7 @@ def construct_gpr_xml(parent, expression):
 def annotate_cobra_from_sbml(cobra_element, sbml_element):
     sbo_term = sbml_element.get("sboTerm")
     if sbo_term is not None:
-        cobra_element.annotation["SBO"] = sbo_term
+        cobra_element.annotation["sbo"] = sbo_term
     meta_id = get_attrib(sbml_element, "metaid")
     if meta_id is None:
         return
@@ -232,7 +246,7 @@ def annotate_sbml_from_cobra(sbml_element, cobra_element):
     bag = SubElement(SubElement(rdf_desc, ns("bqbiol:is")),
                      ns("rdf:Bag"))
     for provider, identifiers in sorted(iteritems(cobra_element.annotation)):
-        if provider == "SBO":
+        if provider == "sbo":
             set_attrib(sbml_element, "sboTerm", identifiers)
             continue
         if isinstance(identifiers, string_types):
@@ -307,7 +321,7 @@ def parse_xml_into_model(xml, number=float):
             reaction.upper_bound = bounds[ub_id]
             reaction.lower_bound = bounds[lb_id]
         except KeyError as e:
-            raise CobraSBMLError("No constant bound with id '%s'" % e.message)
+            raise CobraSBMLError("No constant bound with id '%s'" % str(e))
         reactions.append(reaction)
 
         stoichiometry = defaultdict(lambda: 0)
@@ -340,7 +354,7 @@ def parse_xml_into_model(xml, number=float):
         # set gene reaction rule
         gpr_xml = sbml_reaction.find(GPR_TAG)
         if gpr_xml is not None and len(gpr_xml) != 1:
-            warn("ignoring invalid geneAssocation for " + repr(reaction))
+            warn("ignoring invalid geneAssociation for " + repr(reaction))
             gpr_xml = None
         gpr = process_gpr(gpr_xml[0]) if gpr_xml is not None else ''
         # remove outside parenthesis, if any
@@ -348,24 +362,37 @@ def parse_xml_into_model(xml, number=float):
             gpr = gpr[1:-1].strip()
         gpr = gpr.replace(SBML_DOT, ".")
         reaction.gene_reaction_rule = gpr
-    model.add_reactions(reactions)
+    try:
+        model.add_reactions(reactions)
+    except ValueError as e:
+        warn(str(e))
 
     # objective coefficients are handled after all reactions are added
     obj_list = xml_model.find(ns("fbc:listOfObjectives"))
     if obj_list is None:
         warn("listOfObjectives element not found")
         return model
-    target_objective = get_attrib(obj_list, "fbc:activeObjective")
-    obj_query = OBJECTIVES_XPATH % target_objective
+    target_objective_id = get_attrib(obj_list, "fbc:activeObjective")
+    target_objective = obj_list.find(
+        ns("fbc:objective[@fbc:id='{}']".format(target_objective_id)))
+    obj_direction_long = get_attrib(target_objective, "fbc:type")
+    obj_direction = LONG_SHORT_DIRECTION[obj_direction_long]
+
+    obj_query = OBJECTIVES_XPATH % target_objective_id
+    coefficients = {}
     for sbml_objective in obj_list.findall(obj_query):
         rxn_id = clip(get_attrib(sbml_objective, "fbc:reaction"), "R_")
         try:
             objective_reaction = model.reactions.get_by_id(rxn_id)
-        except KeyError as e:
+        except KeyError:
             raise CobraSBMLError("Objective reaction '%s' not found" % rxn_id)
-        objective_reaction.objective_coefficient = \
-            get_attrib(sbml_objective, "fbc:coefficient", type=number)
-
+        try:
+            coefficients[objective_reaction] = get_attrib(
+                sbml_objective, "fbc:coefficient", type=number)
+        except ValueError as e:
+            warn(str(e))
+    set_objective(model, coefficients)
+    model.solver.objective.direction = obj_direction
     return model
 
 
@@ -398,7 +425,8 @@ def model_to_xml(cobra_model, units=True):
     set_attrib(obj_list_tmp, "fbc:activeObjective", "obj")
     obj_list_tmp = SubElement(obj_list_tmp, ns("fbc:objective"))
     set_attrib(obj_list_tmp, "fbc:id", "obj")
-    set_attrib(obj_list_tmp, "fbc:type", "maximize")
+    set_attrib(obj_list_tmp, "fbc:type",
+               SHORT_LONG_DIRECTION[cobra_model.objective.direction])
     flux_objectives_list = SubElement(obj_list_tmp,
                                       ns("fbc:listOfFluxObjectives"))
 
@@ -407,13 +435,13 @@ def model_to_xml(cobra_model, units=True):
     param_attr = {"constant": "true"}
     if units:
         param_attr["units"] = "mmol_per_gDW_per_hr"
-    # the most common bounds are the minimum, maxmium, and 0
+    # the most common bounds are the minimum, maximum, and 0
     if len(cobra_model.reactions) > 0:
         min_value = min(cobra_model.reactions.list_attr("lower_bound"))
         max_value = max(cobra_model.reactions.list_attr("upper_bound"))
     else:
-        min_value = -1000
-        max_value = 1000
+        min_value = CONFIGURATION.lower_bound
+        max_value = CONFIGURATION.upper_bound
 
     SubElement(parameter_list, "parameter", value=strnum(min_value),
                id="cobra_default_lb", sboTerm="SBO:0000626", **param_attr)
@@ -465,16 +493,16 @@ def model_to_xml(cobra_model, units=True):
     # add in genes
     if len(cobra_model.genes) > 0:
         genes_list = SubElement(xml_model, GENELIST_TAG)
-    for gene in cobra_model.genes:
-        gene_id = gene.id.replace(".", SBML_DOT)
-        sbml_gene = SubElement(genes_list, GENE_TAG)
-        set_attrib(sbml_gene, "fbc:id", "G_" + gene_id)
-        name = gene.name
-        if name is None or len(name) == 0:
-            name = gene.id
-        set_attrib(sbml_gene, "fbc:label", gene_id)
-        set_attrib(sbml_gene, "fbc:name", gene.name)
-        annotate_sbml_from_cobra(sbml_gene, gene)
+        for gene in cobra_model.genes:
+            gene_id = gene.id.replace(".", SBML_DOT)
+            sbml_gene = SubElement(genes_list, GENE_TAG)
+            set_attrib(sbml_gene, "fbc:id", "G_" + gene_id)
+            name = gene.name
+            if name is None or len(name) == 0:
+                name = gene.id
+            set_attrib(sbml_gene, "fbc:label", gene_id)
+            set_attrib(sbml_gene, "fbc:name", name)
+            annotate_sbml_from_cobra(sbml_gene, gene)
 
     # add in reactions
     reactions_list = SubElement(xml_model, "listOfReactions")
@@ -528,7 +556,7 @@ def model_to_xml(cobra_model, units=True):
             gpr = gpr.replace(".", SBML_DOT)
             gpr_xml = SubElement(sbml_reaction, GPR_TAG)
             try:
-                parsed = parse_gpr(gpr)[0]
+                parsed, _ = parse_gpr(gpr)
                 construct_gpr_xml(gpr_xml, parsed.body)
             except Exception as e:
                 print("failed on '%s' in %s" %
@@ -539,21 +567,24 @@ def model_to_xml(cobra_model, units=True):
 
 
 def read_sbml_model(filename, number=float, **kwargs):
+    if not _with_lxml:
+        warn("Install lxml for faster SBML I/O", ImportWarning)
     xmlfile = parse_stream(filename)
     xml = xmlfile.getroot()
     # use libsbml if not l3v1 with fbc v2
-    if xml.get("level") != "3" or xml.get("version") != "1" or \
-            get_attrib(xml, "fbc:required") is None:
+    use_libsbml = (xml.get("level") != "3" or xml.get("version") != "1" or
+                   get_attrib(xml, "fbc:required") is None)
+    if use_libsbml:
         if libsbml is None:
-            raise Exception("libSBML required for fbc < 2")
+            raise ImportError("libSBML required for fbc < 2")
         # libsbml needs a file string, so write to temp file if a file handle
         if hasattr(filename, "read"):
             with NamedTemporaryFile(suffix=".xml", delete=False) as outfile:
-                xmlfile.write(outfile, encoding="UTF-8")
+                xmlfile.write(outfile, encoding="UTF-8", xml_declaration=True)
             filename = outfile.name
         return read_sbml2(filename, **kwargs)
     try:
-        return parse_xml_into_model(xml, number=number, **kwargs)
+        return parse_xml_into_model(xml, number=number)
     except Exception:
         raise CobraSBMLError(
             "Something went wrong reading the model. You can get a detailed "
@@ -563,7 +594,7 @@ def read_sbml_model(filename, number=float, **kwargs):
 
 id_required = {ns(i) for i in ("sbml:model", "sbml:reaction:", "sbml:species",
                                "fbc:geneProduct", "sbml:compartment",
-                               "sbml:paramter", "sbml:UnitDefinition",
+                               "sbml:parameter", "sbml:UnitDefinition",
                                "fbc:objective")}
 invalid_id_detector = re.compile("|".join(re.escape(i[0]) for i in _renames))
 
@@ -595,8 +626,9 @@ def validate_sbml_model(filename, check_model=True):
     xmlfile = parse_stream(filename)
     xml = xmlfile.getroot()
     # use libsbml if not l3v1 with fbc v2
-    if xml.get("level") != "3" or xml.get("version") != "1" or \
-            get_attrib(xml, "fbc:required") is None:
+    use_libsbml = (xml.get("level") != "3" or xml.get("version") != "1" or
+                   get_attrib(xml, "fbc:required") is None)
+    if use_libsbml:
         raise CobraSBMLError("XML is not SBML level 3 v1 with fbc v2")
 
     errors = {k: [] for k in ("validator", "warnings", "SBML errors", "other")}
@@ -606,12 +638,11 @@ def validate_sbml_model(filename, check_model=True):
 
     # make sure there is exactly one model
     xml_models = xml.findall(ns("sbml:model"))
+    if len(xml_models) == 0:
+        raise CobraSBMLError("No SBML model detected in file")
     if len(xml_models) > 1:
         err("More than 1 SBML model detected in file")
-    elif len(xml_models) == 0:
-        err("No SBML model detected in file")
-    else:
-        xml_model = xml_models[0]
+    xml_model = xml_models[0]
 
     # make sure all sbml id's are valid
     all_ids = set()
@@ -658,10 +689,10 @@ def validate_sbml_model(filename, check_model=True):
             model = parse_xml_into_model(xml)
         except CobraSBMLError as e:
             err(str(e), "SBML errors")
-            return (None, errors)
+            return None, errors
         except Exception as e:
             err(str(e), "other")
-            return (None, errors)
+            return None, errors
     errors["warnings"].extend(str(i.message) for i in warning_list)
 
     # check genes
@@ -673,16 +704,17 @@ def validate_sbml_model(filename, check_model=True):
             err("No gene specfied with id 'G_%s'" % gene.id)
 
     if check_model:
-        errors["validator"].extend(check_reaction_bounds(model))
         errors["validator"].extend(check_metabolite_compartment_formula(model))
 
     return model, errors
 
 
 def write_sbml_model(cobra_model, filename, use_fbc_package=True, **kwargs):
+    if not _with_lxml:
+        warn("Install lxml for faster SBML I/O", ImportWarning)
     if not use_fbc_package:
         if libsbml is None:
-            raise Exception("libSBML required to write non-fbc models")
+            raise ImportError("libSBML required to write non-fbc models")
         write_sbml2(cobra_model, filename, use_fbc_package=False, **kwargs)
         return
     # create xml
