@@ -1,66 +1,17 @@
+# -*- coding: utf-8 -*-
+from __future__ import absolute_import
+
+from itertools import chain
+
+import pytest
+
 from cobra.core import Metabolite, Model, Reaction
 from cobra.manipulation import *
-import pytest
-from .conftest import model, salmonella
 
 
 class TestManipulation:
     """Test functions in cobra.manipulation"""
 
-    def test_canonical_form(self, model):
-        # add G constraint to test
-        g_constr = Metabolite("SUCCt2_2__test_G_constraint")
-        g_constr._constraint_sense = "G"
-        g_constr._bound = 5.0
-        model.reactions.get_by_id("SUCCt2_2").add_metabolites({g_constr: 1})
-        assert abs(model.optimize("maximize").f - 0.855) < 0.001
-        # convert to canonical form
-        model = canonical_form(model)
-        assert abs(model.optimize("maximize").f - 0.855) < 10 ** -3
-
-    def test_canonical_form_minimize(self, model):
-        # make a minimization problem
-        model.reactions.get_by_id("Biomass_Ecoli_core").lower_bound = 0.5
-        for reaction in model.reactions:
-            reaction.objective_coefficient = reaction.id == "GAPD"
-        assert abs(model.optimize("minimize").f - 6.27) < 10 ** -3
-        # convert to canonical form. Convert minimize to maximize
-        model = canonical_form(model, objective_sense="minimize")
-        assert abs(model.optimize("maximize").f + 6.27) < 10 ** -3
-        # lower bounds should now be <= constraints
-        assert model.reactions.get_by_id(
-            "Biomass_Ecoli_core").lower_bound == 0.0
-
-    def test_modify_reversible(self, model):
-        model1 = model.copy()
-        model1.optimize()
-        model2 = model.copy()
-        convert_to_irreversible(model2)
-        model2.optimize()
-        assert abs(model1.solution.f - model2.solution.f) < 10 ** -3
-        revert_to_reversible(model2)
-        model2.optimize()
-        assert abs(model1.solution.f - model2.solution.f) < 10 ** -3
-        # Ensure revert_to_reversible is robust to solutions generated both
-        # before and after reversibility conversion, or not solved at all.
-        model3 = model.copy()
-        model3.optimize()
-        convert_to_irreversible(model3)
-        revert_to_reversible(model3)
-        assert abs(model1.solution.f - model3.solution.f) < 10 ** -3
-        # test reaction where both bounds are negative
-        model4 = model.copy()
-        glc = model4.reactions.get_by_id("EX_glc__D_e")
-        glc.upper_bound = -1
-        convert_to_irreversible(model4)
-        model4.optimize()
-        assert abs(model1.solution.f - model4.solution.f) < 10 ** -3
-        glc_rev = model4.reactions.get_by_id(glc.notes["reflection"])
-        assert glc_rev.lower_bound == 1
-        assert glc.upper_bound == 0
-        revert_to_reversible(model4)
-        assert glc.upper_bound == -1
-
     def test_escape_ids(self, model):
         model.reactions.PGI.gene_reaction_rule = "a.b or c"
         assert "a.b" in model.genes
@@ -69,12 +20,15 @@ class TestManipulation:
 
     def test_rename_gene(self, model):
         original_name = model.genes.b1241.name
-        rename_dict = {"b1241": "foo", "hello": "world",
+        rename_dict = {"b1241": "foo", "hello": "world", "b3115": "b3115",
                        "b2465": "b3919", "bar": "2935"}
         modify.rename_genes(model, rename_dict)
-        for i in rename_dict:
-            assert i not in model.genes
+        for i in rename_dict.keys():
+            if i not in rename_dict.values():
+                assert i not in model.genes
+        assert "b3115" in model.genes
         assert "foo" in model.genes
+        assert "world" not in model.genes
         # make sure the object name was preserved
         assert model.genes.foo.name == original_name
         # make sure the reactions are correct
@@ -207,49 +161,75 @@ class TestManipulation:
         fake_DM.add_metabolites({model.metabolites.get_by_id("h_c"): -1})
         # this exchange will be set wrong. The function should not overwrite
         # an existing SBO annotation
-        rxns.get_by_id("EX_h_e").annotation["SBO"] = "SBO:0000628"
+        rxns.get_by_id("EX_h_e").annotation["sbo"] = "SBO:0000628"
         add_SBO(model)
-        assert rxns.EX_o2_e.annotation["SBO"] == "SBO:0000627"
-        assert rxns.DM_h_c.annotation["SBO"] == "SBO:0000628"
-        assert rxns.EX_h_e.annotation["SBO"] == "SBO:0000628"
-
-    def test_validate_reaction_bounds(self, model):
-        model.reactions[0].lower_bound = float("-inf")
-        model.reactions[1].lower_bound = float("nan")
-        model.reactions[0].upper_bound = float("inf")
-        model.reactions[1].upper_bound = float("nan")
-        errors = check_reaction_bounds(model)
-        assert len(errors) == 4
+        assert rxns.EX_o2_e.annotation["sbo"] == "SBO:0000627"
+        assert rxns.DM_h_c.annotation["sbo"] == "SBO:0000628"
+        assert rxns.EX_h_e.annotation["sbo"] == "SBO:0000628"
 
     def test_validate_formula_compartment(self, model):
-        model.metabolites[1].compartment = "fake"
         model.metabolites[1].formula = "(a*.bcde)"
         errors = check_metabolite_compartment_formula(model)
-        assert len(errors) == 2
+        assert len(errors) == 1
 
     def test_validate_mass_balance(self, model):
         assert len(check_mass_balance(model)) == 0
         # if we remove the SBO term which marks the reaction as
         # mass balanced, then the reaction should be detected as
         # no longer mass balanced
-        EX_rxn = model.reactions.query("EX")[0]
-        EX_rxn.annotation.pop("SBO")
+        EX_rxn = model.reactions.query(lambda r: r.boundary)[0]
+        EX_rxn.annotation.pop("sbo")
         balance = check_mass_balance(model)
         assert len(balance) == 1
         assert EX_rxn in balance
         m1 = Metabolite('m1', formula='()')
         r1 = Reaction('r1')
         r1.add_metabolites({m1: 1})
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError), pytest.warns(UserWarning):
             r1.check_mass_balance()
 
-    def test_prune_unused(self, model):
+    def test_prune_unused_mets_output_type(self, model):
+        # test that the output contains metabolite objects
         metabolite = model.metabolites.ru5p__D_c
         [model.reactions.get_by_id(x).remove_from_model() for x in
          ['RPI', 'RPE', 'GND']]
-        unused = delete.prune_unused_metabolites(model)
-        assert unused[0] is metabolite
+        model_pruned, unused = delete.prune_unused_metabolites(model)
+        assert isinstance(model_pruned, Model)
+        assert isinstance(unused[0], Metabolite)
+
+    def test_prune_unused_mets_functionality(self, model):
+        # test that the unused metabolites are not used in the model
+        metabolite1 = model.metabolites.ru5p__D_c
+        metabolite2 = model.metabolites.akg_e
+        metabolite3 = model.metabolites.akg_c
+        reactions = set(chain(metabolite1.reactions,
+                              metabolite2.reactions,
+                              metabolite3.reactions))
+        model.remove_reactions(reactions)
+        model_pruned, unused = delete.prune_unused_metabolites(model)
+        assert metabolite1 in model.metabolites
+        assert metabolite2 in model.metabolites
+        assert metabolite3 in model.metabolites
+        assert metabolite1 not in model_pruned.metabolites
+        assert metabolite2 not in model_pruned.metabolites
+        assert metabolite3 not in model_pruned.metabolites
+
+    def test_prune_unused_rxns_output_type(self, model):
+        # test that the output contains reaction objects
         reaction = Reaction('foo')
         model.add_reaction(reaction)
-        unused = delete.prune_unused_reactions(model)
-        assert unused[0] is reaction
+        model_pruned, unused = delete.prune_unused_reactions(model)
+        assert isinstance(model_pruned, Model)
+        assert isinstance(unused[0], Reaction)
+
+    def test_prune_unused_rxns_functionality(self, model):
+        # test that the unused reactions are not used in the model
+        for x in ['foo1', 'foo2', 'foo3']:
+            model.add_reaction(Reaction(x))
+        model_pruned, unused = delete.prune_unused_reactions(model)
+        assert 'foo1' in model.reactions
+        assert 'foo2' in model.reactions
+        assert 'foo3' in model.reactions
+        assert 'foo1' not in model_pruned.reactions
+        assert 'foo2' not in model_pruned.reactions
+        assert 'foo3' not in model_pruned.reactions
